searchData={"items":[{"type":"behaviour","title":"Unleash","doc":"Unofficial Elixir SDK for the [Unleash Feature Flag System](https://www.getunleash.io/).\n\nUse it to check feature flags and variants via the main usage macros:\n\n    require Unleash.Macros, as: Unleash\n    Unleash.enabled?(:some_feature, fallback: false)\n    Unleash.get_variant(:some_feature_with_variants)\n\nThe SDK additionally provides a bunch of utilities to automatically set some context fields\nin Plug applications (see `Unleash.Plug`) and some utilities for propagation of context,\nfeature flags overrides and impressions across services (see `Unleash.Propagation`).\nNote that Propagation is not a standard Unleash mechanism, but a custom extension\ndefined and developed by [Fresha](https://www.fresha.com/careers).\n\nThis module just contains the \"interface \"of the SDK, meaning the `Unleash` behaviour\nand types. For the actual runtime implementation, check `Unleash.Runtime` and related modules.","ref":"Unleash.html"},{"type":"callback","title":"Unleash.enabled?/2","doc":"Checks if the given feature is enabled.","ref":"Unleash.html#c:enabled?/2"},{"type":"callback","title":"Unleash.get_variant/2","doc":"Returns the variant for the given feature flag.","ref":"Unleash.html#c:get_variant/2"},{"type":"type","title":"Unleash.context/0","doc":"The Unleash Context for activation strategies evaluation. See `Unleash.Context`.","ref":"Unleash.html#t:context/0"},{"type":"type","title":"Unleash.enabled_opt/0","doc":"Option that can be passed to `enabled?/2`.\n\n* `:allow_overrides` - Whether propagated overrides should be considered for the feature flag check.\n  See `t:Unleash.overrides/0`. The default is `false`.\n* `:context` - Context that might be used to evaluate activation strategies. The default is an empty map.\n* `:fallback` - Fallback value that will be returned if the feature is not found in the local cache or overrides.\n  The default is `false`. You can use `nil` if you'd like to handle missing features explicitly.","ref":"Unleash.html#t:enabled_opt/0"},{"type":"type","title":"Unleash.feature_name/0","doc":"Type that can be used to reference a feature.\n\nNote that `:feature_a` and `\"feature_a\"` reference the same feature.","ref":"Unleash.html#t:feature_name/0"},{"type":"type","title":"Unleash.get_variant_opt/0","doc":"Option that can be passed to `get_variant/2`.\n\n* `:context` - Context that might be used to evaluate activation strategies. The default is an empty map.\n* `:fallback` - Fallback value that will be returned if the feature is not found in the local cache.\n  The default is the special \"disabled\" variant.\n  You can use `nil` if you'd like to handle missing features explicitly.","ref":"Unleash.html#t:get_variant_opt/0"},{"type":"type","title":"Unleash.impression/0","doc":"Record of a feature flag activation check.\n\nIn other words, whenever a feature flag is queried for status via `enabled?/3`,\nan impression represents the feature that was checked and the result of that check.\nSee https://docs.getunleash.io/reference/impression-data","ref":"Unleash.html#t:impression/0"},{"type":"type","title":"Unleash.overrides/0","doc":"Represents local feature overrides that can be used to force a feature evaluation value,\nbypassing its activation strategies.\n\nThese can be inherited from the clients,\nsee `Unleash.Propagation` for a high-level introduction to the topic.","ref":"Unleash.html#t:overrides/0"},{"type":"module","title":"Unleash.Cache","doc":"This module is a cache backed by an ETS table. We use it to allow for multiple\nthreads to read the feature flag values concurrently on top of minimizing\nnetwork calls","ref":"Unleash.Cache.html"},{"type":"function","title":"Unleash.Cache.cache_table_name/0","doc":"","ref":"Unleash.Cache.html#cache_table_name/0"},{"type":"function","title":"Unleash.Cache.get_all_feature_names/1","doc":"Will return all values currently stored in the cache","ref":"Unleash.Cache.html#get_all_feature_names/1"},{"type":"function","title":"Unleash.Cache.get_feature/2","doc":"Will return the feature for the given name stored in the cache","ref":"Unleash.Cache.html#get_feature/2"},{"type":"function","title":"Unleash.Cache.get_features/1","doc":"Will return all features stored in the cache","ref":"Unleash.Cache.html#get_features/1"},{"type":"function","title":"Unleash.Cache.init/2","doc":"Will create a new ETS table named `:unleash_cache`","ref":"Unleash.Cache.html#init/2"},{"type":"function","title":"Unleash.Cache.upsert_features/2","doc":"Will upsert (create or update) the given features in the cache\n\nThis will clear the existing peristed features to prevent stale reads","ref":"Unleash.Cache.html#upsert_features/2"},{"type":"behaviour","title":"Unleash.Client","doc":"HTTP Client responsible for communicating with the Unleash API Server.","ref":"Unleash.Client.html"},{"type":"function","title":"Unleash.Client.features/1","doc":"","ref":"Unleash.Client.html#features/1"},{"type":"callback","title":"Unleash.Client.features/1","doc":"","ref":"Unleash.Client.html#c:features/1"},{"type":"function","title":"Unleash.Client.metrics/1","doc":"","ref":"Unleash.Client.html#metrics/1"},{"type":"callback","title":"Unleash.Client.metrics/1","doc":"","ref":"Unleash.Client.html#c:metrics/1"},{"type":"function","title":"Unleash.Client.register/1","doc":"","ref":"Unleash.Client.html#register/1"},{"type":"function","title":"Unleash.Client.register_client/0","doc":"","ref":"Unleash.Client.html#register_client/0"},{"type":"callback","title":"Unleash.Client.register_client/0","doc":"","ref":"Unleash.Client.html#c:register_client/0"},{"type":"function","title":"Unleash.Client.telemetry_metadata/1","doc":"","ref":"Unleash.Client.html#telemetry_metadata/1"},{"type":"type","title":"Unleash.Client.features_result/0","doc":"Type representing the result of attempting to fetch features from the Unleash API.\n\nIt can be:\n\n* `{etag, features}` when fetched fresh features from the API and correspondent new HTTP etag.\n* `{nil, reason}` in case of errors.\n* `:cached` in case of HTTP-level caching","ref":"Unleash.Client.html#t:features_result/0"},{"type":"module","title":"Unleash.Context","doc":"The Unleash Context, mostly used to evaluate conditional activation strategies.\nhttps://docs.getunleash.io/reference/unleash-context.\n\nThe Context has a predefined set of fields,\nbut custom context properties can be set in the `:properties` field.\nNote that, for hystorical / backwards-compatibility reasons,\nboth atom and string keys are allowed in the properties map.\nYou are encouraged to use `find/2` to lookup context values in a way\nthat handles fallback to properties and the different style of keys correctly.","ref":"Unleash.Context.html"},{"type":"function","title":"Unleash.Context.atomize_context_key/1","doc":"","ref":"Unleash.Context.html#atomize_context_key/1"},{"type":"function","title":"Unleash.Context.find/2","doc":"Looks for a context value, taking care of some of the nasty details\nlike fallback to `:properties` (a.k.a. custom context values)\nwith support for both atom and string keys.\n\nSpecifically:\n1. If the key is a standard context key (either in its atom or string form),\n   it looks for it in the standard context fields, without fallbacking to\n   properties.\n2. Otherwise, it looks for it in properties, trying both the atomized and the stringified\n   version of the key, starting from the supplied one.\n\nNOTE that this can create atom at runtimes, which is dangerous.\nIt is highly recommended you do not call this function with values\nfrom untrusted / unbounded sources.","ref":"Unleash.Context.html#find/2"},{"type":"function","title":"Unleash.Context.stringify_context_key/1","doc":"","ref":"Unleash.Context.html#stringify_context_key/1"},{"type":"type","title":"Unleash.Context.t/0","doc":"","ref":"Unleash.Context.html#t:t/0"},{"type":"module","title":"Unleash.Macros","doc":"This module offers a macro API over the `Unleash` behaviour,\nand should be used as the main entrypoint over the Unleash SDK.\n\nDoing so has a few advantages:\n* Delegates the actual check to a configurable runtime implementation.\n  This allows, for example, to use a mock implementation in tests,\n  without having to change anything at the call site.\n* Requires the options to be specified at compile-time,\n  making feature checks more clear and explicit (see chapter below for details).\n* Checks which options have been passed, and generates a compilation warning in case of unknown ones:\n\n      require Unleash.Macros, as: Unleash\n      # generates a compilation warning: unknown option `:falback`.\n      Unleash.enabled?(:flag, falback: true)\n\n* More compile-time checks might be added in the future","ref":"Unleash.Macros.html"},{"type":"module","title":"Options as named arguments - Unleash.Macros","doc":"Whenever you call `enabled?/2` or `get_variant/2`,\noption **keys** must be provided statically at compile-time.\nBasically, you should treat the options as named function arguments.\n\nThis requirement does not extend to the feature name or option values,\nwhich can be dynamic runtime terms -- however, for readability and clarity\nof feature flags checks, it is recommended to also provide those statically whenever possible,\nparticularly for feature names.\n\nFor instance, rather than writing (which would raise a compile time error if using the `enabled?/2` macro):\n\n    iex> opts = [allow_overrides: true, fallback: true]\n    ...> {name, opts} = case some_condition do\n    ...>   true -> {:flag1, Keyword.put(opts, :context, %{user_id: \"u-42\"})}\n    ...>   false -> {:flag2, opts}\n    ...> end\n    ...> Unleash.enabled?(name, opts)\n\nYou should write:\n\n    iex> case some_condition do\n    ...>   true -> Unleash.enabled?(:flag1, allow_overrides: true, fallback: true, context: %{user_id: \"u-42\"})\n    ...>   false -> Unleash.enabled?(:flag2, allow_overrides: true, fallback: true)\n    ...> end\n\nIn the second example, it is much clearer to see which feature flags are being checked and the parameters being used.","ref":"Unleash.Macros.html#module-options-as-named-arguments"},{"type":"macro","title":"Unleash.Macros.enabled?/2","doc":"Checks if the given feature flag is enabled.\n\nThis is a macro that does some compile-time checks\nand delegates to the configured Runtime's `c:Unleash.enabled?/2` at runtime.\nSee `Unleash.Runtime.enabled?/2` for the full documentation.","ref":"Unleash.Macros.html#enabled?/2"},{"type":"macro","title":"Examples - Unleash.Macros.enabled?/2","doc":"iex> require Unleash.Macros, as: Unleash\n    ...> Unleash.enabled?(:flag_a)\n    ...> Unleash.enabled?(:flag_b, context: get_context(), fallback: true, allow_overrides: false)","ref":"Unleash.Macros.html#enabled?/2-examples"},{"type":"macro","title":"Unleash.Macros.get_variant/2","doc":"Returns the variant for the given feature flag.\n\nThis is a macro that does some compile-time checks\nand delegates to the configured Runtime's `c:Unleash.get_variant/2` at runtime.\nSee `Unleash.Runtime.get_variant/2` for the full documentation.","ref":"Unleash.Macros.html#get_variant/2"},{"type":"macro","title":"Examples - Unleash.Macros.get_variant/2","doc":"iex> require Unleash.Macros, as: Unleash\n    ...> Unleash.get_variant(:feature_a)\n    ...> Unleash.get_variant(:test, fallback: %{enabled: true, name: \"variant_two\"})","ref":"Unleash.Macros.html#get_variant/2-examples"},{"type":"module","title":"Unleash.Metrics","doc":"GenServer responsible for reporting metrics to the Unleash API Server.\n\nhttps://docs.getunleash.io/client-specification#metrics","ref":"Unleash.Metrics.html"},{"type":"function","title":"Unleash.Metrics.add_metric/2","doc":"","ref":"Unleash.Metrics.html#add_metric/2"},{"type":"function","title":"Unleash.Metrics.add_variant_metric/2","doc":"","ref":"Unleash.Metrics.html#add_variant_metric/2"},{"type":"function","title":"Unleash.Metrics.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Unleash.Metrics.html#child_spec/1"},{"type":"function","title":"Unleash.Metrics.init/1","doc":"","ref":"Unleash.Metrics.html#init/1"},{"type":"function","title":"Unleash.Metrics.start_link/1","doc":"","ref":"Unleash.Metrics.html#start_link/1"},{"type":"function","title":"Unleash.Metrics.start_link/2","doc":"","ref":"Unleash.Metrics.html#start_link/2"},{"type":"type","title":"Unleash.Metrics.t/0","doc":"","ref":"Unleash.Metrics.html#t:t/0"},{"type":"module","title":"Unleash.Plug","doc":"An extra fancy `Plug` and utility functions to help when developing `Plug`\nor `Phoenix`-based applications. It automatically puts together a\n`t:Unleash.context/0` and stores it in the `t:Plug.Conn.t()`'s `assigns`.\n\nTo use, call `plug Unleash.Plug` in your plug pipeline. It depends on the\nsession, and requires being after `:fetch_session` to work. It accepts the\nfollowing options:\n\n* `:user_id`: The key under which the user's ID is found in the session.\n* `:session_id`: The key under wwhich the session ID is found in the\n  session.\n\nAfter which, `enabled?/3` is usable.\n\nIf you are _also_ using `PhoenixGon` in your application, you can call\n`put_feature/3` to put a specific feature flag into the gon object for\nusing on the client side.","ref":"Unleash.Plug.html"},{"type":"function","title":"Unleash.Plug.enabled?/3","doc":"Given a `t:Plug.Conn.t/0`, a feature, and (optionally) a boolean, return\nwhether or not a feature is enabled. This requires this plug to be a part\nof the plug pipeline, as it will construct an `t:Unleash.context()/0` out\nof the session.","ref":"Unleash.Plug.html#enabled?/3"},{"type":"function","title":"Examples - Unleash.Plug.enabled?/3","doc":"iex> Unleash.Plug.enabled?(conn, :test)\n    false\n\n    iex> Unleash.Plug.enabled?(conn, :test, true)\n    true","ref":"Unleash.Plug.html#enabled?/3-examples"},{"type":"function","title":"Unleash.Plug.put_feature/3","doc":"If you are using `PhoenixGon`, you can call this to put a feature in the\ngon object to be used on the client side. It will be available under\n`window.Gon.getAssets('features')`. It listens to the options that\nare configured by `PhoenixGon.Pipeline`.","ref":"Unleash.Plug.html#put_feature/3"},{"type":"function","title":"Examples - Unleash.Plug.put_feature/3","doc":"iex> Unleash.Plug.put_feature(conn, :test)\n    %Plug.Conn{}\n\n    iex> Unleash.Plug.enabled?(conn, :test, true)\n    true","ref":"Unleash.Plug.html#put_feature/3-examples"},{"type":"module","title":"Unleash.Propagation","doc":"The Unleash Propagation mechanism enables the propagation of feature flag contexts,\noverrides, and impressions throughout a distributed system,\nsimilarly to how OpenTelemetry helps propagate Trace Contexts to build distributed traces.","ref":"Unleash.Propagation.html"},{"type":"module","title":"Downstream Propagation (Context and Overrides) - Unleash.Propagation","doc":"Downstream propagation allows you to set unleash context and overrides which will automatically\napply to any subsequent calls to `Unleash.enabled?/2` and `Unleash.get_variant/2` calls\nwithin the same process or its descendants.\n\n- `put_context/1`: Sets a context which will be used to enrich the local context when evaluating\n  activation strategies of a feature toggle.\n  The local context has precendence and its values will never be overwritten by the propagated context.\n- `put_overrides/1`: Sets feature flags overrides that take precedence over normal feature evaluations.\n\nFor Plug-based applications (such as Phoenix apps), you can use the Plugs provided in\n`Unleash.Propagation.Plugs` to automatically extract context and overrides from request headers.\nTo further propagate them when calling external downstream services, you can retrieve them\nusing `get_context/0` and `get_overrides/0`, and use the `Unleash.Propagation.Serialization` module\nto serialize them to request headers.","ref":"Unleash.Propagation.html#module-downstream-propagation-context-and-overrides"},{"type":"module","title":"Upstream Propagation (Impressions) - Unleash.Propagation","doc":"This mechanism captures data on which feature flags are being checked during a request by the current process\nand its descendants. This information can be returned to the caller (e.g., included in response headers).\n\n- `track_impressions/0`: Begins tracking feature flag checks for the current process and descendants,\n  returning a reference to the tracking session.\n- `impressions/0`: Retrieves the list of tracked impressions.\n  The impressions are deduplicated based on (feature_name, status); meaning you could have up to 2\n  impressions on the same feature if it has been queried twice with different results\n  (which should be quite rare).\n\nImpression tracking has limited support for variants: impressions are recorded when a feature is queried\nvia `get_variant/3`, but only the `enabled: boolean()` status will be recorded, without any information\non the returned variant.","ref":"Unleash.Propagation.html#module-upstream-propagation-impressions"},{"type":"module","title":"Scoping of the Propagation Mechanism - Unleash.Propagation","doc":"Both the context and impressions tracking mechanisms are scoped to the process tree branch,\nensuring data is confined to the current process and its descendants.\nThis helps manage and contain the state within logical execution boundaries, like a request lifecycle.\nThe SDK uses [ProcessTree](https://hexdocs.pm/process_tree/ProcessTree.html) to achieve this,\nso its limitations regarding tracking of parent processes apply.\n\nFor global, unscoped, feature flag impressions tracking, you should not use the impressions features\nprovided by this module, but you should instead hook into the telemetry events emitted by the SDK:\n- `[:unleash, :feature, :enabled]`\n- `[:unleash, :variant, :get]`","ref":"Unleash.Propagation.html#module-scoping-of-the-propagation-mechanism"},{"type":"function","title":"Unleash.Propagation.get_context/0","doc":"","ref":"Unleash.Propagation.html#get_context/0"},{"type":"function","title":"Unleash.Propagation.get_overrides/0","doc":"","ref":"Unleash.Propagation.html#get_overrides/0"},{"type":"function","title":"Unleash.Propagation.import_impressions/1","doc":"","ref":"Unleash.Propagation.html#import_impressions/1"},{"type":"function","title":"Unleash.Propagation.impressions/0","doc":"Returns the impressions generated within this tracking session,\ni.e. since `track_impressions/0` was called by the current process or one of its ancestors.\n\nReturns `nil` if no impression tracking session is active.\n\nSee `track_impressions/0` for more details.","ref":"Unleash.Propagation.html#impressions/0"},{"type":"function","title":"Unleash.Propagation.put_context/1","doc":"","ref":"Unleash.Propagation.html#put_context/1"},{"type":"function","title":"Unleash.Propagation.put_overrides/1","doc":"","ref":"Unleash.Propagation.html#put_overrides/1"},{"type":"function","title":"Unleash.Propagation.record_impression/2","doc":"Records an impression in the current tracking session.\nDoes nothing if there's no active tracking session.\n\nThis function is used by `Unleash`, typically you should not call this directly.\n\nSee `track_impressions/0` for more details.","ref":"Unleash.Propagation.html#record_impression/2"},{"type":"function","title":"Unleash.Propagation.track_impressions/0","doc":"Starts tracking impressions generated by the current process and descendant processes.\n\nReturns `{:ok, ref}` where `ref` is an opaque reference to this specific tracking session,\nor `{:error, {:already_tracking, ref}}` if there was a preexisting tracking session for\nthe current process.\n\nThe recorded impressions can then be retrieved by calling `impressions/0`.\n\nSee `t:Unleash.impression/0` for a brief introduction to Unleash impressions.\n\nNote that to prevent unbounded memory usage, this function should be called\nfrom a process with a limited lifetime, e.g. a request handler.\nAll the recorded impressions are scoped to the calling process,\nand they are deleted and garbage-collected when the process dies.","ref":"Unleash.Propagation.html#track_impressions/0"},{"type":"opaque","title":"Unleash.Propagation.tracking_session/0","doc":"Opaque reference to an impressions tracking session.","ref":"Unleash.Propagation.html#t:tracking_session/0"},{"type":"module","title":"Unleash.Repo","doc":"This genserver polls the unleash service each time the given interval has\nelapsed, refreshing both our local ETS cache and the backup state file if the\nflag state has diverged.\n\nThe following configuration values are used:\n\nConfig.features_period(): polling interval - default 15 seconds\nConfig.retries(): number of time the call to refresh values is allowed to retry - default -1 (0)","ref":"Unleash.Repo.html"},{"type":"function","title":"Unleash.Repo.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Unleash.Repo.html#child_spec/1"},{"type":"function","title":"Unleash.Repo.get_all_feature_names/0","doc":"","ref":"Unleash.Repo.html#get_all_feature_names/0"},{"type":"function","title":"Unleash.Repo.get_feature/1","doc":"","ref":"Unleash.Repo.html#get_feature/1"},{"type":"function","title":"Unleash.Repo.init/1","doc":"","ref":"Unleash.Repo.html#init/1"},{"type":"function","title":"Unleash.Repo.start_link/1","doc":"","ref":"Unleash.Repo.html#start_link/1"},{"type":"function","title":"Unleash.Repo.start_polling/0","doc":"","ref":"Unleash.Repo.html#start_polling/0"},{"type":"module","title":"Unleash.Runtime","doc":"Main Runtime implementation for the Unleash SDK.\n\nLike all the official SDKs, this one keeps and serves feature from a local cache\n(in our case, based on an `:ets` table), including evaluating activation strategies locally.\nThe features are initialised from a bootstrap file and updated periodically\nfrom the Unleash API in the background, independently of how many feature checks are invoked.\n\nThis allows applications using this SDK to get the status of a feature instantaneously and locally,\nwithout overloading the Unleash Server with API calls; but it also means that the features might be stale,\nup to the polling interval or even longer in case of errors contacting the Unleash APIs.\n\nThis Runtime is an Elixir Application that comes with its own Supervision Tree,\nwhich starts two GenServer worker processes:\n\n* `Unleash.Repo`, responsible for periodically refreshing the features from the Unleash API Server\nand maintaining the local cache updated.\n* `Unleash.Metrics`, responsible for periodically\n[sending metrics](https://docs.getunleash.io/client-specification#metrics)\nto the Unleash API Server.\n\nThe latter can be disabled with `disable_metrics: true`; while setting `disable_client: true`\nwill disable both of them and the SDK will start in \"disconnected mode\",\nwhere no features are defined and feature flags values are returned only from\nlocal overrides (see `Unleash.Propagation`) and fallback values.\n\nNote that:\n* Currently, in \"disconnected mode\", the SDK won't even load features from the local bootstrap file.\nThat might change in the future, which would allow the SDK to work with a static set of\nlocal features initialised from a local file.\n* In the test environment, the SDK won't automatically start polling from the API, even if the client is enabled.\nYou'll have to start it explicitly with `Unleash.Repo.start_polling/0`.","ref":"Unleash.Runtime.html"},{"type":"function","title":"Unleash.Runtime.enabled?/2","doc":"Checks if the given feature is enabled.\n\nTakes a feature name and a keyword list of options,\nwhich are used to customise the behaviour of the feature check.\nSee `t:Unleash.enabled_opt/0` for the supported options and their defaults.\n\nReturns a boolean indicating whether the feature flag is enabled.\nCan return `nil` only if `:fallback` was set to `nil` and the feature was not found locally.\n\nNOTE: Prefer using `Unleash.Macros.enabled?/2` whenever possible. See `Unleash.Macros`.","ref":"Unleash.Runtime.html#enabled?/2"},{"type":"function","title":"Unleash.Runtime.get_variant/2","doc":"Returns the variant for the given feature flag.\n\nStrategy variants are prioritised. If none is available then it will fallback to using feature\nvariants.\nFor more info on strategy variants see https://docs.getunleash.io/reference/strategy-variants.\nFeature variants are deprecated, but are still supported for backwards compatibility and you can\nfind more info about them here: https://docs.getunleash.io/reference/feature-toggle-variants\n\nIt takes a feature name and a keyword list of options, which are used to customise the behaviour\nof the feature check. See `t:Unleash.get_variant_opt/0` for the supported options and their\ndefaults.\n\nReturns a `t:Unleash.Variant.t/0` representing the selected variant.\n\nA special variant `%{enabled: false, name: \"disabled\"}` will be returned in\nany of these scenarios:\n* the feature flag is disabled\n* the feature flag does not define any strategy variants in the active strategy\n* the feature flag does not define any feature variants\n* the feature flag does not exist and no fallback has been supplied\n\nNOTE: Prefer using `Unleash.Macros.get_variant/2` whenever possible. See `Unleash.Macros`.","ref":"Unleash.Runtime.html#get_variant/2"},{"type":"function","title":"Unleash.Runtime.start/2","doc":"Starts the Unleash SDK Runtime.\n\nSee `Unleash.Runtime` for details.","ref":"Unleash.Runtime.html#start/2"},{"type":"behaviour","title":"Unleash.Strategies","doc":"Used to get all available strategies in the client. You can add your\nstrategies to the client by extending this module:\n\n```elixir\ndefmodule MyApp.Strategies do\n  @behaviour Unleash.Strategies\n\n  def strategies, do: Unleash.Strategies.strategies() ++ []\nend\n```\n\nand then setting it in your configuration:\n\n```elixir\nconfig :unleash_fresha, Unleash, strategies: MyApp.Strategies\n```","ref":"Unleash.Strategies.html"},{"type":"function","title":"Unleash.Strategies.strategies/0","doc":"Returns all the strategies that are supported by this client. They can be\nviewed under `Strategy`.\n\nFor completeness, the list is:\n\n* `userWithId`: `Unleash.Strategy.ActiveForUsersWithId`\n* `applicationHostname`: `Unleash.Strategy.ApplicationHostname`\n* `default`: `Unleash.Strategy.Default`\n* `gradualRolloutRandom`: `Unleash.Strategy.GradualRolloutRandom`\n* `gradualRolloutSessionId`: `Unleash.Strategy.GradualRolloutSessionId`\n* `gradualRolloutUserId`: `Unleash.Strategy.GradualRolloutUserId`\n* `remoteAddress`: `Unleash.Strategy.RemoteAddress`","ref":"Unleash.Strategies.html#strategies/0"},{"type":"callback","title":"Unleash.Strategies.strategies/0","doc":"Should return a list of all the avilable strategies in the format\n`{\"name\", Module}`. The name must match the name of the strategy in the\nUnleash server, and the module must implement the `Unleash.Strategy` behaviour.","ref":"Unleash.Strategies.html#c:strategies/0"},{"type":"behaviour","title":"Unleash.Strategy","doc":"Used to extend the client and create custom strategies. To do so, `use` this\nmodule within your custom strategy and implmenent `c:enabled?/2`. Provide a\nname that is human-readable, as it is logged.\n\n```elixir\ndefmodule MyApp.CustomStrategy\n  use Unleash.Strategy, name: \"CustomStrategy\"\n\n  def enabled?(_params, _context), do: true\nend\n```","ref":"Unleash.Strategy.html"},{"type":"callback","title":"Unleash.Strategy.enabled?/2","doc":"You can implement this callback a couple of ways, returning a bare `boolean()`\nor a `{boolean, map()}`. The latter is preferred, as it generates a\n`:debug` level log entry detailing the name of the strategy, the result, and\nthe contents of `map()`, in an effort to help understand why the result was\nwhat it was.","ref":"Unleash.Strategy.html#c:enabled?/2"},{"type":"callback","title":"Arguments - Unleash.Strategy.enabled?/2","doc":"* `parameters` - A map of parameters returned from the Unleash server. This\n  can be whatever you like, such as a configured list of `userIds`.\n* `context` - The context passed into `Unleash.enabled?/2`.","ref":"Unleash.Strategy.html#c:enabled?/2-arguments"},{"type":"callback","title":"Examples - Unleash.Strategy.enabled?/2","doc":"```elixir\n@behaviour Unleash.Strategy\n\ndef enabled?(params, context), do: {false, params}\n\ndef enabled(params, %{}), do: false\n```","ref":"Unleash.Strategy.html#c:enabled?/2-examples"},{"type":"function","title":"Unleash.Strategy.from_map/0","doc":"","ref":"Unleash.Strategy.html#from_map/0"},{"type":"function","title":"Unleash.Strategy.from_map/1","doc":"","ref":"Unleash.Strategy.html#from_map/1"},{"type":"module","title":"Unleash.Variant","doc":"Defines types and functions for representing and working with strategy and feature variants.\n\n  Variants extend feature flags beyond simple on/off states, enabling multiple return values for\n  more granular feature control.\n\n  Feature flags are designed to let you decide which users have access to a feature. Variants are\n  designed to let you decide which version of the feature are user gets access to.","ref":"Unleash.Variant.html"},{"type":"module","title":"Overview - Unleash.Variant","doc":"While a basic feature flag exists in either on or off states, variants allow for multiple\n  possible states within an enabled feature. Variants can be defined:\n  * Within strategies (recommended approach)\n  * Directly on the feature flag (deprecated)\n\n  Variants have a weight which determines the likelihood of users receiving a specific variant.","ref":"Unleash.Variant.html#module-overview"},{"type":"module","title":"Common Use Case - Unleash.Variant","doc":"A typical pattern involves:\n  1. Using a feature flag to control access to an entire feature\n  2. Applying strategies to target specific user segments\n  3. Using variants to customise the feature experience for different segments\n\n  For example, you might use strategies to determine which user segments can access a new UI\n  component, then use variants to show different button colors to different segments within that\n  group for A/B testing.","ref":"Unleash.Variant.html#module-common-use-case"},{"type":"module","title":"Documentation - Unleash.Variant","doc":"* [Strategy variants (recommended)](https://docs.getunleash.io/reference/strategy-variants)\n  * [Feature variants (deprecated)](https://docs.getunleash.io/reference/feature-toggle-variants)","ref":"Unleash.Variant.html#module-documentation"},{"type":"function","title":"Unleash.Variant.disabled/0","doc":"","ref":"Unleash.Variant.html#disabled/0"},{"type":"function","title":"Unleash.Variant.from_map/1","doc":"","ref":"Unleash.Variant.html#from_map/1"},{"type":"function","title":"Unleash.Variant.select_feature_variant/2","doc":"","ref":"Unleash.Variant.html#select_feature_variant/2"},{"type":"function","title":"Unleash.Variant.select_strategy_variant/3","doc":"","ref":"Unleash.Variant.html#select_strategy_variant/3"},{"type":"function","title":"Unleash.Variant.to_map/1","doc":"","ref":"Unleash.Variant.html#to_map/1"},{"type":"function","title":"Unleash.Variant.to_map/2","doc":"","ref":"Unleash.Variant.html#to_map/2"},{"type":"type","title":"Unleash.Variant.result/0","doc":"","ref":"Unleash.Variant.html#t:result/0"},{"type":"type","title":"Unleash.Variant.t/0","doc":"","ref":"Unleash.Variant.html#t:t/0"},{"type":"module","title":"Unleash.Propagation.Plugs","doc":"Module that defines plugs to automatically integrate with the `Unleash.Propagation` mechanism.\n\nAll you have to do is plug them in your scope, pipeline, router or endpoint:\n\n    defmodule MyAppRouter do\n      use Plug.Router\n      import Unleash.Propagation.Plugs\n\n      plug :extract_unleash_context\n      plug :extract_unleash_overrides\n      plug :track_impressions\n\n      # Rest of your plugs...\n    end\n\nYou then can simply use `Unleash` as usual in your controllers,\nand all propagation features (client-supplied context, client-supplied overrides and\nreturning feature toggles impressions) will automagically work!\n\nSee `Unleash.Propagation` for more details.","ref":"Unleash.Propagation.Plugs.html"},{"type":"function","title":"Unleash.Propagation.Plugs.extract_header_and_store/2","doc":"A generic Plug to extract an header value, transform it, and store it.\n\nIt's unlikely you'll need to use this directly, and you should simply plug\n`extract_unleash_context/2` and `extract_unleash_overrides/2` in your pipelines instead,\nbut if you know what you're doing and you need custom behaviour, feel free to use this.","ref":"Unleash.Propagation.Plugs.html#extract_header_and_store/2"},{"type":"function","title":"Unleash.Propagation.Plugs.extract_unleash_context/2","doc":"A Plug that extracts the unleash context from a request header and stores it in the\nprocess dictionary under `:unleash_context`.\n\nBy default it uses `x-unleash-context` header, but you can specify the `header_name` option\nto use a different one:\n\n    plug :extract_unleash_context, header_name: \"my-custom-header\"","ref":"Unleash.Propagation.Plugs.html#extract_unleash_context/2"},{"type":"function","title":"Unleash.Propagation.Plugs.extract_unleash_overrides/2","doc":"A Plug that extracts unleash overrides from a request header and stores them in the\nprocess dictionary under `:unleash_overrides`.\n\nBy default it uses `x-unleash-overrides` header, but you can specify the `header_name` option\nto use a different one:\n\n    plug :extract_unleash_overrides, header_name: \"my-custom-header\"","ref":"Unleash.Propagation.Plugs.html#extract_unleash_overrides/2"},{"type":"function","title":"Unleash.Propagation.Plugs.track_impressions/2","doc":"A Plug that will make your controller track all of the feature toggles invoked\nwhile handling the request (including spawned subprocesses)\nand return the (deduplicated) impressions in a response header.\n\nBy default it uses `x-unleash-impressions` header, but you can specify the `header_name` option\nto use a different one:\n\n    plug :track_impressions, header_name: \"my-custom-header\"","ref":"Unleash.Propagation.Plugs.html#track_impressions/2"},{"type":"module","title":"Unleash.Propagation.Serialization","doc":"Module handling the (de)serialization of Unleash objects for propagation.\nSee `Unleash.Propagation`.","ref":"Unleash.Propagation.Serialization.html"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_context/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_context/1"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_context!/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_context!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_impressions/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_impressions/1"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_impressions!/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_impressions!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_overrides/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_overrides/1"},{"type":"function","title":"Unleash.Propagation.Serialization.deserialize_overrides!/1","doc":"","ref":"Unleash.Propagation.Serialization.html#deserialize_overrides!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.serialize_context!/1","doc":"Serializes context for further propagation to downstream services.\n\nNote that only propagation-friendly context fields are serialized.","ref":"Unleash.Propagation.Serialization.html#serialize_context!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.serialize_impressions!/1","doc":"","ref":"Unleash.Propagation.Serialization.html#serialize_impressions!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.serialize_overrides!/1","doc":"Serializes overrides for further propagation to downstream services.","ref":"Unleash.Propagation.Serialization.html#serialize_overrides!/1"},{"type":"function","title":"Unleash.Propagation.Serialization.validate_context_properties/1","doc":"","ref":"Unleash.Propagation.Serialization.html#validate_context_properties/1"},{"type":"module","title":"Unleash.Propagation.GRPC.ContextClientInterceptor","doc":"A gRPC Client Interceptor designed to propagate Unleash context and overrides downstream.\n\nThe context and overrides must be established in the propagation context,\neither manually (using `Unleash.Propagation.put_context/1`)\nor through middleware, such as\n`Unleash.Propagation.GRPC.ContextClientInterceptor` or\n`Unleash.Propagation.Plugs.extract_unleash_context/2`.\n\nFor further details on the propagation mechanism, see `Unleash.Propagation`.\n\nNote that this interceptor will raise if it finds invalid context or overrides values\nin the propagation context that it can't serialize. This can happen if you put values\nin the propagation context manually and you do so incorrectly.\n\nIf you'd like to prevent such instances from breaking your gRPC requests,\nyou can easily build your own interceptor that wraps this one and catches errors, for example:\n\n    defmodule MyUnleashContextClientInterceptorWrapper do\n      @moduledoc false\n      @behaviour GRPC.ClientInterceptor\n\n      @impl true\n      def init(_), do: nil\n\n      def call(stream, req, next, opts) do\n        Unleash.Propagation.GRPC.ContextClientInterceptor.call(stream, req, next, opts)\n      rescue\n        err ->\n          Logger.error(\"your error message here\")\n          next.(stream, req)  # Continue the gRPC request processing\n      end\n    end","ref":"Unleash.Propagation.GRPC.ContextClientInterceptor.html"},{"type":"module","title":"Unleash.Propagation.GRPC.ContextServerInterceptor","doc":"A GRPC Server Interceptor to extract unleash context and overrides\nfrom incoming GRPC request headers and store them for feature flag checks\nand further propagation downstream (e.g. via `Unleash.Propagation.GRPC.ContextClientInterceptor`).\n\nSee `Unleash.Propagation` for more details on the propagation mechanism.\n\nNote that, contrarily to the ContextClientInterceptor, this interceptor does not raise\nin case of invalid context or overrides in the gRPC request headers.\nThe difference is due to the nature of the data: in here, it's dynamic input data,\nbeyond our control. Invalid data is expected and we should handle it gracefully.","ref":"Unleash.Propagation.GRPC.ContextServerInterceptor.html"},{"type":"module","title":"Unleash.Propagation.GRPC.ImpressionsClientInterceptor","doc":"a gRPC Client Interceptor to automatically backpropagate impressions\nreturned by downstream gRPC services.\nClient-side counterpart of `Unleash.Propagation.GRPC.ImpressionsServerInterceptor`.\n\nThe impressions returned by the downstream gRPC service will be imported\nand merged with the impressions generated locally.\n\nNOTE:\n* Due to elxir-grpc Client Interceptor limitations,\n  this interceptor will save the downstream-returned impressions\n  only if the downstream is called with the option `return_headers: true` (See `GRPC.Stub`).\n  See https://github.com/elixir-grpc/grpc/issues/388\n* This interceptor won't save downstream-returned impressions\n  if no impression tracking session is active when calling the downstream service.\n  See `Unleash.Propagation.track_impressions()`\n\nFor further details on the propagation mechanism, see `Unleash.Propagation`.","ref":"Unleash.Propagation.GRPC.ImpressionsClientInterceptor.html"},{"type":"module","title":"Unleash.Propagation.GRPC.ImpressionsServerInterceptor","doc":"A GRPC Server Interceptor to automatically track feature impressions\nwhile handling the request, and return them back to the caller in the response headers.\n\nSee `Unleash.Propagation` for more details on the propagation mechanism.","ref":"Unleash.Propagation.GRPC.ImpressionsServerInterceptor.html"},{"type":"module","title":"Unleash.Strategy.ActiveForUsersWithId","doc":"Requires `:user_id` in `t:Unleash.context/0`\n\nBased on the\n[`userWithId`](https://unleash.github.io/docs/activation_strategy#userwithid)\nstrategy.","ref":"Unleash.Strategy.ActiveForUsersWithId.html"},{"type":"module","title":"Unleash.Strategy.ApplicationHostname","doc":"Does not require anything in `t:Unleash.context/0`\n\nBased on the\n[`applicationHostname`](https://unleash.github.io/docs/activation_strategy#applicationhostname)\nstrategy.","ref":"Unleash.Strategy.ApplicationHostname.html"},{"type":"function","title":"Unleash.Strategy.ApplicationHostname.enabled?/2","doc":"","ref":"Unleash.Strategy.ApplicationHostname.html#enabled?/2"},{"type":"module","title":"Unleash.Strategy.Constraint","doc":"Module that is used to verify\n[constraints](https://www.unleash-hosted.com/docs/strategy-constraints/) are\nmet.\n\nThese constraints allow for very complex and specifc strategies to be\nenacted by allowing users to specify context values to include or exclude.\n\n- DATE_AFTER\n- DATE_BEFORE\n- IN\n- NOT_IN\n- NUM_EQ\n- NUM_GT\n- NUM_GTE\n- NUM_LT\n- NUM_LTE\n- SEMVER_EQ\n- SEMVER_GT\n- SEMVER_LT\n- STR_CONTAINS\n- STR_ENDS_WITH\n- STR_STARTS_WITH","ref":"Unleash.Strategy.Constraint.html"},{"type":"function","title":"Unleash.Strategy.Constraint.verify_all/2","doc":"","ref":"Unleash.Strategy.Constraint.html#verify_all/2"},{"type":"module","title":"Unleash.Strategy.Default","doc":"Does not require anything in `t:Unleash.context/0`\n\nBased on the\n[`default`](https://unleash.github.io/docs/activation_strategy#default)\nstrategy.","ref":"Unleash.Strategy.Default.html"},{"type":"function","title":"Unleash.Strategy.Default.enabled?/2","doc":"","ref":"Unleash.Strategy.Default.html#enabled?/2"},{"type":"module","title":"Unleash.Strategy.FlexibleRollout","doc":"Can depend on `:user_id` or `:session_id` in `t:Unleash.context/0`\n\nBased on the\n[`flexibleRollout`](https://unleash.github.io/docs/activation_strategy#flexiblerollout)\nstrategy.","ref":"Unleash.Strategy.FlexibleRollout.html"},{"type":"function","title":"Unleash.Strategy.FlexibleRollout.enabled?/2","doc":"","ref":"Unleash.Strategy.FlexibleRollout.html#enabled?/2"},{"type":"module","title":"Unleash.Strategy.GradualRolloutRandom","doc":"Does not require anything in `t:Unleash.context/0`\n\nBased on the\n[`gradualRollourRandom`](https://unleash.github.io/docs/activation_strategy#gradualrolloutrandom)\nstrategy.","ref":"Unleash.Strategy.GradualRolloutRandom.html"},{"type":"module","title":"Unleash.Strategy.GradualRolloutSessionId","doc":"Requires `:session_id` in `t:Unleash.context/0`\n\nBased on the\n[`gradualRolloutSessionId`](https://unleash.github.io/docs/activation_strategy#gradualrolloutsessionid)\nstrategy.","ref":"Unleash.Strategy.GradualRolloutSessionId.html"},{"type":"function","title":"Unleash.Strategy.GradualRolloutSessionId.enabled?/2","doc":"","ref":"Unleash.Strategy.GradualRolloutSessionId.html#enabled?/2"},{"type":"module","title":"Unleash.Strategy.GradualRolloutUserId","doc":"Requires `:user_id` in `t:Unleash.context/0`\n\nBased on the\n[`gradualRolloutUserId`](https://unleash.github.io/docs/activation_strategy#gradualrolloutuserid)\nstrategy.","ref":"Unleash.Strategy.GradualRolloutUserId.html"},{"type":"function","title":"Unleash.Strategy.GradualRolloutUserId.enabled?/2","doc":"","ref":"Unleash.Strategy.GradualRolloutUserId.html#enabled?/2"},{"type":"module","title":"Unleash.Strategy.RemoteAddress","doc":"Requires `:remote_address` in `t:Unleash.context/0`\n\nBased on the\n[`remoteAddress`](https://unleash.github.io/docs/activation_strategy#remoteaddress)\nstrategy.","ref":"Unleash.Strategy.RemoteAddress.html"},{"type":"module","title":"Unleash.Strategy.Utils","doc":"Utilities that might help you create your own strategies.\n\nAutomatically `alias`ed when `use`ing `Unleash.Strategy`","ref":"Unleash.Strategy.Utils.html"},{"type":"function","title":"Unleash.Strategy.Utils.in_list?/3","doc":"Given a comma-separated list and a member, check to see if the member is in\nthe list. An optional transformation function can be passed in to apply\nto the items in the list.","ref":"Unleash.Strategy.Utils.html#in_list?/3"},{"type":"function","title":"Examples - Unleash.Strategy.Utils.in_list?/3","doc":"iex> in_list?(\"a,B,c,D\", \"b\", &String.downcase/1)\n    true\n\n    iex> in_list?(\"a,B,c,D\", \"b\")\n    false","ref":"Unleash.Strategy.Utils.html#in_list?/3-examples"},{"type":"function","title":"Unleash.Strategy.Utils.normalize/3","doc":"Given an ID and group ID, normalize them using a `Murmur` hash. Used in all\nstrategies that provide \"stickiness\".\n\nSee the\n[`gradualRolloutUserId`](https://unleash.github.io/docs/activation_strategy#gradualrolloutuserid)\nstrategy.","ref":"Unleash.Strategy.Utils.html#normalize/3"},{"type":"function","title":"Unleash.Strategy.Utils.normalize_variant/3","doc":"","ref":"Unleash.Strategy.Utils.html#normalize_variant/3"},{"type":"function","title":"Unleash.Strategy.Utils.parse_int/1","doc":"Parse a string to a non-negative integer, returning only the number\nor 0 if it isn't just a number.","ref":"Unleash.Strategy.Utils.html#parse_int/1"},{"type":"function","title":"Examples - Unleash.Strategy.Utils.parse_int/1","doc":"iex> parse_int(\"5\")\n    5\n\n    iex> parse_int(\"5ab23a\")\n    0\n\n    iex> parse_int(5)\n    5","ref":"Unleash.Strategy.Utils.html#parse_int/1-examples"},{"type":"exception","title":"Unleash.Propagation.Serialization.ContextDeserializationError","doc":"An error representing context deserialization failure.","ref":"Unleash.Propagation.Serialization.ContextDeserializationError.html"},{"type":"type","title":"Unleash.Propagation.Serialization.ContextDeserializationError.t/0","doc":"","ref":"Unleash.Propagation.Serialization.ContextDeserializationError.html#t:t/0"},{"type":"exception","title":"Unleash.Propagation.Serialization.ContextSerializationError","doc":"An error representing context serialization failure.","ref":"Unleash.Propagation.Serialization.ContextSerializationError.html"},{"type":"type","title":"Unleash.Propagation.Serialization.ContextSerializationError.t/0","doc":"","ref":"Unleash.Propagation.Serialization.ContextSerializationError.html#t:t/0"},{"type":"exception","title":"Unleash.Propagation.Serialization.ImpressionsDeserializationError","doc":"An error representing impressions deserialization failure.","ref":"Unleash.Propagation.Serialization.ImpressionsDeserializationError.html"},{"type":"type","title":"Unleash.Propagation.Serialization.ImpressionsDeserializationError.t/0","doc":"","ref":"Unleash.Propagation.Serialization.ImpressionsDeserializationError.html#t:t/0"},{"type":"exception","title":"Unleash.Propagation.Serialization.OverridesDeserializationError","doc":"An error representing overrides deserialization failure.","ref":"Unleash.Propagation.Serialization.OverridesDeserializationError.html"},{"type":"type","title":"Unleash.Propagation.Serialization.OverridesDeserializationError.t/0","doc":"","ref":"Unleash.Propagation.Serialization.OverridesDeserializationError.html#t:t/0"},{"type":"exception","title":"Unleash.Propagation.Serialization.OverridesSerializationError","doc":"An error representing overrides serialization failure.","ref":"Unleash.Propagation.Serialization.OverridesSerializationError.html"},{"type":"type","title":"Unleash.Propagation.Serialization.OverridesSerializationError.t/0","doc":"","ref":"Unleash.Propagation.Serialization.OverridesSerializationError.html#t:t/0"},{"type":"extras","title":"Unleash SDK (Fresha Edition)","doc":"# Unleash SDK (Fresha Edition)\nAn Elixir SDK for the [Unleash Feature Flag System](https://www.getunleash.io/).\n\nOriginally started as a fork of [unleash_ex](https://gitlab.com/afontaine/unleash_ex/),\nbut since then it has diverged quite a bit,\nmost notably by replacing [Mojito](https://github.com/appcues/mojito) with [Req](https://github.com/wojtekmach/req),\nchanging its usage APIs in version 2.0.0 and by implementing custom extensions like\n[Propagation](https://hexdocs.pm/unleash_fresha/Unleash.Propagation.html).\n\nTake a look at the [CHANGELOG](./CHANGELOG.md), the two projects diverged on version 1.9.0.","ref":"readme.html"},{"type":"extras","title":"Installation - Unleash SDK (Fresha Edition)","doc":"Add `unleash_fresha` to your dependendencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:unleash_fresha, \"~> 2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Unleash SDK (Fresha Edition)","doc":"Use `Unleash.Macros.enabled?/2` (or `/1`) and `Unleash.Macros.get_variant/2` (or `/1`) to check if a feature\nis enabled and get the right variant, in case of features with variants.\n\n```elixir\n# You have to require the `Unleash.Macros` module.\n# In this example we alias it to `Unleash` to make it nicer, but you do you.\niex> require Unleash.Macros, as: Unleash\n\niex> Unleash.enabled(:some_feature, fallback: false, context: %{user_id: \"u-123\"})\nfalse\n\niex> Unleash.get_variant(:some_feature_with_variants, context: %{properties: %{\"lucky\" => \"yes\"}})\n%{enabled: true, name: \"variant_c\", payload: %{}}\n```\n\nThe macros do some static checks at compile-time for safer usage,\nand allow you to swap the runtime implementation that does the feature check,\nfor example to use a mock one in the test environment (see next section).\n\nIf for some reason you can't use the macros, you can call the runtime equivalents in `Unleash.Runtime` directly,\nbut by doing so you lose the benefits mentioned above.\n\nCheck the moduledocs of `Unleash`, `Unleash.Runtime` and `Unleash.Macros` for more details.","ref":"readme.html#usage"},{"type":"extras","title":"Testing of code that uses feature flags - Unleash SDK (Fresha Edition)","doc":"In tests, you might not want to connect to an Unleash Server, and/or you might want to\nforce feature flag values to test specific code branches.\n\nYou can do so by swapping the SDK to use a different Runtime in tests,\ne.g. a mock one via [`Mox`](https://github.com/dashbitco/mox).\n\nAs an example:\n\n```elixir\n# in config/test.exs\nconfig :unleash_fresha, Unleash, runtime: Unleash.MockRuntime\n\n# Somewhere in your test support files\nMox.defmock(Unleash.MockRuntime, for: Unleash)\n\n# Your code can use Unleash as usual\ndefmodule SomeModule do\n  require Unleash.Macros, as: Unleash\n  \n  def some_function() do\n    case Unleash.enabled?(:some_flag) do\n      true -> :yes\n      false -> :no\n    end\n  end\nend\n\n# In your tests, set expectations or stubs for feature flag checks\ntest \"returns :yes when flag is enabled\" do\n  # Note that the Runtime functions only have the 2-arity clauses,\n  # if you invoke the 1-arity macro, `opts` will be the default empty keyword list.\n  Mox.expect(Unleash.MockRuntime, :enabled?, 1, fn :some_flag, _opts -> true end)\n  \n  assert :yes == SomeModule.some_function()\nend\n```\n\nThis is just one possible strategy.\nAs another example, you could define a Runtime with a fixed behaviour instead of explicit per-test\nexpectations.","ref":"readme.html#testing-of-code-that-uses-feature-flags"},{"type":"extras","title":"Migrating from `unleash_ex` - Unleash SDK (Fresha Edition)","doc":"- change the dependency name and version in mix.exs\n- rename `:uneash` to `:unleash_fresha` in config files\n- replace calls to functions `is_enabled/1,2,3`, `enabled?/1,2,3` and `get_variant/1,2,3` to the\n  correspondent invokations of the macros `enabled?/1,2` and `get_variant/1,2`.  \n  See the [CHANGELOG](./CHANGELOG.md) for version 2.0.0 and the docs of the macros for more details.\n- use any of the extra features provided by `unleash_fresha`, e.g. the gRPC interceptors.","ref":"readme.html#migrating-from-unleash_ex"},{"type":"extras","title":"Configuration - Unleash SDK (Fresha Edition)","doc":"There are many configuration options available, and they are listed below with\ntheir defaults. These go into the relevant `config/*.exs` file.\n\n```elixir\nconfig :unleash_fresha, Unleash,\n  url: \"\", # The URL of the Unleash server to connect to, should include up to http://base.url/api\n  appname: \"\", # The app name, used for registration\n  instance_id: \"\", # The instance ID, used for metrics tracking\n  metrics_period: 10 * 60 * 1000, # Send metrics every 10 minutes, in milliseconds\n  features_period: 15 * 1000, # Poll for new flags every 15 seconds, in milliseconds\n  strategies: Unleash.Strategies, # Which module to request for toggle strategies\n  backup_file: nil, # Backup file in the event that contacting the server fails\n  custom_http_headers: [], # A keyword list of custom headers to send to the server\n  runtime: Unleash.Runtime, # the Runtime module which will be used for feature flags checks\n  disable_client: false, # Whether or not to enable the client\n  disable_metrics: false, # Whether or not to send metrics,\n  retries: -1 # How many times to retry on failure, -1 disables limit\n  app_env: :dev # Which environment we're in\n```\n\n`:custom_http_headers` should follow [the format prescribed by Req](https://hexdocs.pm/req/Req.html#module-header-names).\n\n`:strategies` should be a module that implements\n`c:Unleash.Strategies.strategies/0`. See [Extensibility](#extensibility)\nfor more information.","ref":"readme.html#configuration"},{"type":"extras","title":"Extensibility - Unleash SDK (Fresha Edition)","doc":"If you need to create your own strategies, you can extend the `Unleash` client\nby implementing the callbacks in both `Unleash.Strategy` and\n`Unleash.Strategies` as well as passing your new `Strategies` module in as\nconfiguration:\n\n1. Create your new strategy. See `Unleash.Strategy` for details on the correct\n    API and `Unleash.Strategy.Utils` for helpful utilities.\n\n    ```elixir\n    defmodule MyApp.Strategy.Environment do\n      use Unleash.Strategy\n\n      def enabled?(%{\"environments\" => environments}, _context) do\n        with {:ok, environment} <- MyApp.Config.get_environment(),\n             environment = List.to_string(environment) do\n          {Utils.in_list?(environments, environment, &String.downcase/1),\n           %{environment: environment, environments: environments}}\n        end\n      end\n    end\n    ```\n\n1. Create a new strategies module. See `Unleash.Strategies` for details on the correct\n    API.\n\n    ```elixir\n    defmodule MyApp.Strategies do\n      @behaviour Unleash.Strategies\n\n      def strategies do\n        [{\"environment\", MyApp.Strategy.Environment}] ++ Unleash.Strategies.strateges()\n      end\n    end\n    ```\n\n1. Configure your application to use your new strategies list.\n\n    ```elixir\n    config :unleash_fresha, Unleash, strategies: MyApp.Strategies\n    ```","ref":"readme.html#extensibility"},{"type":"extras","title":"Telemetry events - Unleash SDK (Fresha Edition)","doc":"From Unleash 1.9, telemetry events  are emitted by the Unleash client\nlibrary. You can attach to these events and collect metrics or use the `Logger`,\nfor example:\n\n```elixir\n# An example of checking if Unleash server is reachable during the periodic\n# features fetch.\n:ok =\n  :telemetry.attach_many(\n    :duffel_core_feature_heatbeat_metric,\n    [\n      [:unleash, :client, :fetch_features, :stop],\n      [:unleash, :client, :fetch_features, :exception]\n    ],\n    fn [:unleash, :client, :fetch_features, action],\n        _measurements,\n        metadata,\n        _config ->\n      require Logger\n\n      http_status = metadata[:http_response_status]\n\n      if action == :stop and http_status in [200, 304] do\n        Logger.info(\"Fetching features are ok\")\n      else\n        Logger.info(\"Error on fetching features!!!\")\n      end\n    end,\n    %{}\n  )\n```\n\nThe following events are emitted by the Unleash library:\n\n| Event | When | Measurement | Metadata |\n| --- | --- | --- | --- |\n| `[:unleash, :feature, :enabled?, :start]` | dispatched by Unleash whenever a feature state has been requested.| `%{system_time: system_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), feature: String.t()}` |\n| `[:unleash, :feature, :enabled?, :stop]` | dispatched by Unleash whenever a feature check has successfully returned a result. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), feature: String.t(), result: boolean() | nil, reason: atom(), strategy_evaluations: [{String.t(), boolean()}], feature_enabled: boolean()}` |\n| `[:unleash, :feature, :enabled?, :exception]` | dispatched by Unleash after exceptions on fetching a feature's activation state. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), feature: String.t(), kind: :throw \\| :error \\| :exit, reason: term(), stacktrace: Exception.stacktrace()}` |\n\n| Event | When | Measurement | Metadata |\n| --- | --- | --- | --- |\n| `[:unleash, :client, :fetch_features, :start]` | dispatched by Unleash.Client whenever it start to fetch features from a remote Unleash server. | `%{system_time: system_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), etag: String.t() \\| nil, url: String.t()}` |\n| `[:unleash, :client, :fetch_features, :stop]`| dispatched by Unleash.Client whenever it finishes to fetch features from a remote Unleash server.  | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), etag: String.t() \\| nil, url: String.t(), http_response_status: pos_integer \\| nil, error: struct() \\| nil}` |\n| `[:unleash, :client, :fetch_features, :exception]` | dispatched by Unleash.Client after exceptions on fetching features. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), etag: String.t() \\| nil, url: String.t(), kind: :throw \\| :error \\| :exit, reason: term(), stacktrace: Exception.stacktrace()}` |\n\n| Event | When | Measurement | Metadata |\n| --- | --- | --- | --- |\n| `[:unleash, :client, :register, :start]` | dispatched by Unleash.Client whenever it starts to register in an Unleash server. | `%{system_time: system_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), sdk_version: String.t(), strategies: [String.t()], interval: pos_integer}` |\n| `[:unleash, :client, :register, :stop]` | dispatched by Unleash.Client whenever it finishes to register in an Unleash server. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), sdk_version: String.t(), strategies: [String.t()], interval: pos_integer, http_response_status: pos_integer \\| nil, error: struct() \\| nil}` |\n| `[:unleash, :client, :register, :exception]` | dispatched by Unleash.Client after exceptions on registering in an Unleash server. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), sdk_version: String.t(), strategies: [String.t()], interval: pos_integer, kind: :throw \\| :error \\| :exit, reason: term(), stacktrace: Exception.stacktrace()}` |\n\n| Event | When | Measurement | Metadata |\n| --- | --- | --- | --- |\n| `[:unleash, :client, :push_metrics, :start]` | dispatched by Unleash.Client whenever it starts to push metrics to an Unleash server. | `%{system_time: system_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), metrics_payload: %{ :bucket => %{:start => String.t(), :stop => String.t(), toggles: %{ String.t() => %{ :yes => pos_integer(), :no => pos_integer() } } } } }` |\n| `[:unleash, :client, :push_metrics, :stop]` | dispatched by Unleash.Client whenever it finishes to push metrics to an Unleash server. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), http_response_status: pos_integer \\| nil, error: struct() \\| nil, metrics_payload: %{ :bucket => %{:start => String.t(), :stop => String.t(), toggles: %{ String.t() => %{ :yes => pos_integer(), :no => pos_integer() } } } } }` |\n| `[:unleash, :client, :push_metrics, :exception]` | dispatched by Unleash.Client after exceptions on pushing metrics to an Unleash server. | `%{duration: native_time, monotonic_time: monotonic_time}` | `%{appname: String.t(), instance_id: String.t(), url: String.t(), kind: :throw \\| :error \\| :exit, reason: term(), stacktrace: Exception.stacktrace(), metrics_payload: %{ :bucket => %{:start => String.t(), :stop => String.t(), toggles: %{ String.t() => %{ :yes => pos_integer(), :no => pos_integer() } } } } }`\n\n| Event                                     | When                           | Metadata                                                                                                         |\n| ----------------------------------------- | ------------------------------ | ---------------------------------------------------------------------------------------------------------------- |\n| `[:unleash, :repo, :schedule]`            | dispatched by Unleash.Repo when scheduling a poll to the server for metrics                                                                                                                                                                 | `%{appname: String.t(), instance_id: String.t(), retries: integer(), etag: String.t(), interval: pos_integer()}` |\n| `[:unleash, :repo, :backup_file_update]`  | dispatched by Unleash.Repo when it writes features to the backup file.                                                                                                                                                                      | `%{appname: String.t(), instance_id: String.t(), content: String.t(), filename: String.t()}`                     |\n| `[:unleash, :repo, :disable_polling]`     | dispatched by Unleash.Repo when polling gets disabled due to retries running out or zero retries being specified initially.                                                                                                                 | `%{appname: String.t(), instance_id: String.t(), retries: integer(), etag: String.t()}`                          |\n| `[:unleash, :repo, :features_update]`     | dispatched by Unleash.Repo when features are updated.                                                                                                                                                                                       | `%{appname: String.t(), instance_id: String.t(), retries: integer(), etag: String.t(), source: :remote \\| :cache \\| :backup_file}` |","ref":"readme.html#telemetry-events"},{"type":"extras","title":"Testing - Unleash SDK (Fresha Edition)","doc":"Tests are using upstream [client-specification](https://github.com/Unleash/client-specification) which needs to be cloned to priv folder","ref":"readme.html#testing"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n<!-- %% CHANGELOG_ENTRIES %% -->","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"3.0.0 - Changelog","doc":"**WARNING: BREAKING CHANGE**\nUsers relying on propagated overrides for variants will be affected by this\nupdate.","ref":"changelog.html#3-0-0"},{"type":"extras","title":"Highlights - Changelog","doc":"* Updated get_variant/2 to support the official Unleash API's strategy variants\n  while maintaining compatibility with feature variants.\n* API interface remains unchanged.\n* Removed support for propagated overrides for variants.","ref":"changelog.html#highlights"},{"type":"extras","title":"Details - Changelog","doc":"The official Unleash API has deprecated feature variants in favor of strategy\nvariants. We've adapted our implementation to align with this change while\nensuring backward compatibility.\n\n#### Why we removed propagated overrides for variants\n\n* Supporting propagated overrides for strategy variants would require knowledge\n  of the specific strategy where each variant is defined.\n* This would significantly complicate both the overrides header schema and\n  implementation.\n* The existing implementation for feature variants overrides lacked robust\n  support for creating overrides with payloads.\n\nThis architectural decision balances API modernization with maintaining a clean\nimplementation.","ref":"changelog.html#details"},{"type":"extras","title":"2.1.0 - Changelog","doc":"* Restore Fresha-specific behaviour of automatically populating a custom\n  context property `\"namespace\"` from the environment variable `UNLEASH_NAMESPACE`.\n* Remove references to Mojito. It has been swapped out for Req in version 1.10.0.","ref":"changelog.html#2-1-0"},{"type":"extras","title":"2.0.0 - 2024-08-15 - Changelog","doc":"","ref":"changelog.html#2-0-0-2024-08-15"},{"type":"extras","title":"Highlights - Changelog","doc":"#### New API & Upgrade Guide\n**WARNING: BREAKING CHANGES!**\n\nThe main APIs of the SDK have been restructured for safety, clarity and expressiveness,\nand the main utilities to do feature checks are now the two macros `enabled?/2` and `get_variant/2`,\nwhich feature a keyword-based call signature.\n\nOld usage:\n```elixir\nUnleash.enabled?(:my_flag)\n# Second argument here is the \"default\" (fallback if feature is missing)\nUnleash.enabled?(\"another_flag\", false)\n# Second argument here is the context that gets passed to feature activation strategy evaluation\nUnleash.enabled?(:flag_three, %{user_id: \"user-123\"})\n# 3-arity clause\nUnleash.enabled?(\"flag_four\", %{}, true)\n```\n\nNew usage:\n```elixir\n# You have to require the `Unleash.Macros` module.\n# In this example we alias it to `Unleash` to make it nicer, but you do you.\nrequire Unleash.Macros, as: Unleash\n\nUnleash.enabled?(:my_flag)\n# Explicit named arguments avoid ambiguous calls and allow extra\n# options to be introduced in the future.\nUnleash.enabled?(\"another_flag\", fallback: false)\nUnleash.enabled?(:flag_three, context: %{user_id: \"user-123\"})\n# Note that you can pass the keyword arguments in any order\nUnleash.enabled?(\"flag_four\", fallback: true, context: %{})\n\n# Overrides have to be explicitly allowed when evaluating a feature or they will be ignored.\nUnleash.enabled?(\"flag_five\", fallback: nil, allow_overrides: true)\n```\n\nThe macros allow us to check at compile-time what options are provided and emit compilation warnings in case of unsupported ones,\nand do other compile-time checks to ensure the clarity and correctness of feature flags checks.\nAdditionally, by using the macros you can swap out the underlying runtime implementation,\nfor example to use a mock one in tests. See the updated [README](./README.md).\n\nNote that this means the names (not values!) of the parameters must be passed as compile-time constants.\nThis is not valid:\n\n```elixir\ndef my_function(feature_check_options) do\n  # Opts is a local variable at compile-time\n  Unleash.enabled?(:some_flag, feature_check_options)\nend\n\n# Opts is an expression at compile time\nUnleash.enabled?(:some_flag, if(some_condition(), do: [], else: [fallback: ]))\n```\n\nBut don't worry, you're very unlikely to incur in this case in normal usage.\nEven if you do, you will get a helpful compilation error explaining the problem\nand how to address it.\n\nIn edge cases where you can't use the macros, which should be quite unlikely,\nyou can use directly the equivalent functions in `Unleash.Runtime`.","ref":"changelog.html#highlights"},{"type":"extras","title":"Added - Changelog","doc":"* Two new macros `Unleash.Macros.enabled?/1,2` and `Unleash.Macros.get_variant/1,2`,\n  which are now the main methods to check a feature flag.\n* the `Unleash` module now defines a behaviour for Runtime implementations.\n  This can be used to define Mox/Hammox mocks and, combined with the next point,\n  facilitates testing. See the testing section on the [README](./README.md).\n* the actual runtime implementation (and `Application`) has been moved to `Unleash.Runtime`.\n  The Runtime can be swapped via compile-time config.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* It is now possible to pass `nil` as a fallback value to `enabled?/2` and `get_variant/2`,\n    which allows you to customise how you handle missing feature flags.\n* For extra safety, overrides are now \"opt-in\".\n  You have to explicitly tell Unleash to consider them, when checking for a feature/variant,\n  by passing the option `allow_overrides: true`.\n* The `find_in_context/2` function has been moved to `Unleash.Context.find/2`.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"* The following functions have been removed:\n  - `Unleash.is_enabled/1,2,3`\n  - `Unleash.enabled?/1,2,3`\n  - `Unleash.get_variant/1,2,3`","ref":"changelog.html#removed"},{"type":"extras","title":"Improved - Changelog","doc":"* Improved documentation and tests throughout\n* Improved type specs\n* Updated and reworded README.","ref":"changelog.html#improved"},{"type":"extras","title":"1.16.0 - 2024-08-08 - Changelog","doc":"","ref":"changelog.html#1-16-0-2024-08-08"},{"type":"extras","title":"Highlights - Changelog","doc":"#### gRPC Interceptors\n\nThis release introduces gRPC interceptors that seamlessly enable the propagation mechanism\nwithin your gRPC-based applications.\n\nIt includes four interceptors:\n\n- `Unleash.Propagation.GRPC.ContextServerInterceptor`\n- `Unleash.Propagation.GRPC.ContextClientInterceptor`\n- `Unleash.Propagation.GRPC.ImpressionsServerInterceptor`\n- `Unleash.Propagation.GRPC.ImpressionsClientInterceptor`\n\nThe first two interceptors are responsible for the propagation of context and overrides.  \nThe server interceptor automatically extracts the incoming Unleash context and feature overrides,\nwhich are then considered during feature evaluations within the context of request handling.  \nThe client interceptor propagates context and overrides further when invoking downstream gRPC services.\n\nThe latter two interceptors handle the propagation of feature impressions.  \nThe server interceptor initiates an impressions tracking session during request handling and automatically returns the generated impressions when sending the gRPC response.  \nMeanwhile, the client interceptor reads impressions returned by downstream gRPC services and merges them with the locally generated impressions.","ref":"changelog.html#highlights"},{"type":"extras","title":"Added - Changelog","doc":"* gRPC interceptors for the Propagation mechanism.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* Changes to test setup helpers which should strongly reduce the flakiness of the tests.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.15.0 - 2024-08-06 - Changelog","doc":"","ref":"changelog.html#1-15-0-2024-08-06"},{"type":"extras","title":"Highlights - Changelog","doc":"#### Impression Tracking\n\nThis release introduces a new propagation mechanism to track\n[impressions](https://docs.getunleash.io/reference/impression-data) generated\nwithin a tracking session, which typically corresponds to a logical execution unit, such as a request handler.\nThese tracked impressions can be retrieved, for instance, to be sent back to the caller in a response header.\n\nIt includes a new Plug, `track_impression/2`, which can be seamlessly integrated into your router or pipeline\nto handle this process for you.","ref":"changelog.html#highlights"},{"type":"extras","title":"Added - Changelog","doc":"* Impressions tracking core functionality: `Unleash.Propagation.track_impressions/0`,\n  `Unleash.Propagation.record_impression/2`, `Unleash.Propagation.impressions/0`\n* Impressions tracking Plug: `Unleash.Propagation.Plugs.track_impressions/1,2`","ref":"changelog.html#added"},{"type":"extras","title":"1.14.0 - 2024-07-29 - Changelog","doc":"* Add `Unleash.Propagation.Serialization.serialize_context!/1` and\n  `Unleash.Propagation.Serialization.serialize_overrides!/1`.\n\n  These new methods can be invoked by users of the SDK to futher propagate context\n  and overrides when sending requests to downstream services.","ref":"changelog.html#1-14-0-2024-07-29"},{"type":"extras","title":"1.13.0 - 2024-07-29 - Changelog","doc":"* Introduce the Propagation mechanism\n\n  The propagation mechanism allows to set overrides and context values that will be considered\n  whenever a feature state is evaluated (via `enabled/3` or `get_variant/3`) in the current\n  or descendant processes.\n  The SDK includes two new Plugs that can be used in your Plug-based application\n  to automatically extracts overrides and context out of incoming HTTP request headers\n  and store them in the Propagation mechanism.\n\n* Support for both string keys and atom keys in context `properties`.\n\n  Previously the SDK would expect a context's `:properties` field to be a map with atom keys.\n  (though this expectation wasn't enforced nor specified anywhere).\n\n  With the new propagation mechanism, arbitrary properties can be set from HTTP requests.\n  The SDK stores them as string keys instead of doing a potentially dangerous\n  runtime conversion of dynamic data to atoms.\n\n  **This is a potentially breaking change if your application relied on properties\n  with string values being ignored by the SDK** (which is unlikely).\n\n  A new function `Unleash.find_in_context/2` has been introduced to consistently\n  lookup values in a context, which handles fallback to `:properties` and the string/atom dichotomy.","ref":"changelog.html#1-13-0-2024-07-29"},{"type":"extras","title":"1.12.0 - 2024-07-23 - Changelog","doc":"* Tune retry behaviour of HTTP requests to Unleash API server.\n  They are now only retried once after 50ms, instead of the default `Req` behaviour of retrying 3 times after 1s, 2s, 4s.\n* Silence warnings on `Req` retries.\n* Log an error when failing to contact the Unleash API server (after retry).","ref":"changelog.html#1-12-0-2024-07-23"},{"type":"extras","title":"1.11.0 - 2024-07-19 - Changelog","doc":"* Avoid failing and crashing in case of missing or invalid bootstrap/backup file.\n* Load bootstrap/backup file synchronously on startup.\n* Do not read the backup file on server errrors. Keep using the local cache instead, which we assume to be in-sync with the backup file, as they bot get udpated whenever we obtain fresh features from the server.","ref":"changelog.html#1-11-0-2024-07-19"},{"type":"extras","title":"1.10.2 - 2024-06-19 - Changelog","doc":"* Add `namespace` to context","ref":"changelog.html#1-10-2-2024-06-19"},{"type":"extras","title":"1.10.1 - 2024-06-19 - Changelog","doc":"* Fix caseInsensitve","ref":"changelog.html#1-10-1-2024-06-19"},{"type":"extras","title":"1.10.0 - 2024-05-20 - Changelog","doc":"* replace Mojito with Req","ref":"changelog.html#1-10-0-2024-05-20"},{"type":"extras","title":"1.9.2 - 2023-01-25 - Changelog","doc":"* Update dependencies\n* Add version, date, number comparison","ref":"changelog.html#1-9-2-2023-01-25"},{"type":"extras","title":"1.9.0 - 2023-01-25 - Changelog","doc":"* add: Telemetry events for `Unleash.Client` [!28](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/28)\n* add: Telemetry events for `Unleash.enabled?` [!29](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/29)\n* add: Telemetry events for `Unleash.get_variant` [!30](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/30)\n* add: Telemetry events for `Unleash.Repo` [!31](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/31)\n* add: Telemetry events for metrics pushed to server [!33](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/33)\n* remove: Logger calls [!37](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/36)\n\nThanks [Jekri Orlina](https://gitlab.com/jekku) and [Sam Hutchings](https://gitlab.com/samhutchings) for telemetry\nadoption.","ref":"changelog.html#1-9-0-2023-01-25"},{"type":"extras","title":"1.8.3 - 2022-05-31 - Changelog","doc":"* fix: Avoid crash when mojito connect is closed for the metrics client\n  [!26](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/26) Thanks [John Bell](https://gitlab.com/johnabell)!\n* fix: Feature flag without variants cause FunctionClause Error\n  [!25](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/25) Thanks\n  [Ulisses Almeida](https://gitlab.com/ulissesalmeida)","ref":"changelog.html#1-8-3-2022-05-31"},{"type":"extras","title":"1.8.2 - Changelog","doc":"* fix `Unleash.Metrics.add_metrics` ccrash for unrecorded feature_toggle [!24](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/24) Thanks [calvin-kargo](https://gitlab.com/calvin-kargo)!","ref":"changelog.html#1-8-2"},{"type":"extras","title":"1.8.1 - Changelog","doc":"* fix Unleash.Metrics to provide proper start_link/1 behaviour [!23](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/23) Thanks [calvinsadewa](https://gitlab.com/calvin-kargo)!","ref":"changelog.html#1-8-1"},{"type":"extras","title":"1.8.0 - Changelog","doc":"* Use ETS cache to store features rather than Genserver state\n  [!22](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/22) Thanks\n  [Maximilien Rothier Bautzer](https://gitlab.com/cachemoi)","ref":"changelog.html#1-8-0"},{"type":"extras","title":"1.7.2 - Changelog","doc":"* Disabled flags should not produce an active variant [!20](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/20) Thanks [Daniel Cooper](https://gitlab.com/danielcooper)","ref":"changelog.html#1-7-2"},{"type":"extras","title":"1.7.1 - Changelog","doc":"* Handle unexpected mojito_response messages [!19](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/19) Thanks [Daniel Cooper](https://gitlab.com/danielcooper)","ref":"changelog.html#1-7-1"},{"type":"extras","title":"1.7.0 - Changelog","doc":"* Also transform the member in a list query [!17](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/17) Thanks [Daniel Cooper](https://gitlab.com/danielcooper)!\n* Update mojito to current [!18](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/18) Thanks [Daniel Cooper](https://gitlab.com/danielcooper)!","ref":"changelog.html#1-7-0"},{"type":"extras","title":"1.6.0 - Changelog","doc":"* Add Flexible Rollout Strategy [!12](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/12)\n* Support Constraints for Feature Flags [!14](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/14)\n* Avoid using Mix.env at runtime [!16](https://gitlab.com/afontaine/unleash_ex/-/merge_requests/16) Thanks [Daniel Cooper](https://gitlab.com/danielcooper)!","ref":"changelog.html#1-6-0"},{"type":"extras","title":"1.5.0 - Changelog","doc":"* Add ability to fetch all feature flag names [decfa826](https://gitlab.com/afontaine/unleash_ex/commit/decfa826fca2d656a61f8e77c29138ea28214473)","ref":"changelog.html#1-5-0"},{"type":"extras","title":"1.4.1 - Changelog","doc":"* Suport Running With Distillery [!10](https://gitlab.com/afontaine/unleash_ex/merge_requests/10) Thanks [Thomas Chandler](https://gitlab.com/thomaschandler)!\n* Use cached features when unexpected response received from Unleash server [!11](https://gitlab.com/afontaine/unleash_ex/merge_requests/11) Thanks [Thomas Chandler](https://gitlab.com/thomaschandler)!","ref":"changelog.html#1-4-1"},{"type":"extras","title":"1.4.0 - Changelog","doc":"* Add Support for Capturing Metrics on Variants [!7](https://gitlab.com/afontaine/unleash_ex/merge_requests/7)","ref":"changelog.html#1-4-0"},{"type":"extras","title":"1.3.1 - Changelog","doc":"* Fix Missing Content-Type for Posting Metrics [89d073cf](https://gitlab.com/afontaine/unleash_ex/commit/89d073cf6e507816259c8481b9510c56db672deb)","ref":"changelog.html#1-3-1"},{"type":"extras","title":"1.3.0 - Changelog","doc":"* Fix Variant Serialization to JSON [e5f4fd3c](https://gitlab.com/afontaine/unleash_ex/commit/e5f4fd3cece12810afbe789c122404e9169bd1ef)\n* Use Mojito to Pool HTTP Connections [e5f4fd3c](https://gitlab.com/afontaine/unleash_ex/commit/e5f4fd3cece12810afbe789c122404e9169bd1ef)","ref":"changelog.html#1-3-0"},{"type":"extras","title":"1.2.0 - Changelog","doc":"* Add Variant Support [!5](https://gitlab.com/afontaine/unleash_ex/merge_requests/5)\n* Fix ETag Usage [f30b80bf](https://gitlab.com/afontaine/unleash_ex/commit/f30b80bf931f56f5de908ca738977c2e540155e4)","ref":"changelog.html#1-2-0"},{"type":"extras","title":"1.1.0 - Changelog","doc":"* Add Optional `Plug` Module [!6](https://gitlab.com/afontaine/unleash_ex/merge_requests/6)","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.0 - Changelog","doc":"* Implement Client Specification Tests [!4](https://gitlab.com/afontaine/unleash_ex/merge_requests/4)","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.2.0 - Changelog","doc":"* Add E-Tag Caching Support [!1](https://gitlab.com/afontaine/unleash_ex/merge_requests/1)\n* Send the Library Version when Registering Client [!2](https://gitlab.com/afontaine/unleash_ex/merge_requests/2)\n* Add Checks for a Retry Limit [!3](https://gitlab.com/afontaine/unleash_ex/merge_requests/3)","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.0 - Changelog","doc":"* Initial release","ref":"changelog.html#0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}